<?php

declare(strict_types=1);

namespace Dotenv\Exception;

use InvalidArgumentException;

final class InvalidPathException extends InvalidArgumentException implements ExceptionInterface
{
    //
}
